@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# Copyright (C) 2004, 2007, 2009-2013  Internet Systems Consortium, Inc. ("ISC")
+# Copyright (C) 2004, 2007, 2009-2016  Internet Systems Consortium, Inc. ("ISC")
 # Copyright (C) 2000, 2001  Internet Software Consortium.
 #
 # Permission to use, copy, modify, and/or distribute this software for any
@@ -15,193 +15,216 @@
 # OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 # PERFORMANCE OF THIS SOFTWARE.
 
-# $Id$
-
 SYSTEMTESTTOP=..
 . $SYSTEMTESTTOP/conf.sh
 
 status=0
 n=0
 
-echo "I:checking non-cachable NXDOMAIN response handling"
+n=`expr $n + 1`
+echo "I:checking non-cachable NXDOMAIN response handling ($n)"
 ret=0
-$DIG +tcp nxdomain.example.net @10.53.0.1 a -p 5300 > dig.out || ret=1
-grep "status: NXDOMAIN" dig.out > /dev/null || ret=1
+$DIG +tcp nxdomain.example.net @10.53.0.1 a -p 5300 > dig.out.ns1.test${n} || ret=1
+grep "status: NXDOMAIN" dig.out.ns1.test${n} > /dev/null || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
 if [ -x ${SAMPLE} ] ; then
-echo "I:checking non-cachable NXDOMAIN response handling using dns_client"
+   n=`expr $n + 1`
+   echo "I:checking non-cachable NXDOMAIN response handling using dns_client ($n)"
    ret=0
-   ${SAMPLE} -p 5300 -t a 10.53.0.1 nxdomain.example.net 2> sample.out || ret=1
-   grep "resolution failed: ncache nxdomain" sample.out > /dev/null || ret=1
+   ${SAMPLE} -p 5300 -t a 10.53.0.1 nxdomain.example.net 2> sample.out.ns1.test${n} || ret=1
+   grep "resolution failed: ncache nxdomain" sample.out.ns1.test${n} > /dev/null || ret=1
    if [ $ret != 0 ]; then echo "I:failed"; fi
    status=`expr $status + $ret`
 fi
 
-echo "I:checking non-cachable NODATA response handling"
+n=`expr $n + 1`
+echo "I:checking non-cachable NODATA response handling ($n)"
 ret=0
-$DIG +tcp nodata.example.net @10.53.0.1 a -p 5300 > dig.out || ret=1
-grep "status: NOERROR" dig.out > /dev/null || ret=1
+$DIG +tcp nodata.example.net @10.53.0.1 a -p 5300 > dig.out.ns1.test${n} || ret=1
+grep "status: NOERROR" dig.out.ns1.test${n} > /dev/null || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
 if [ -x ${SAMPLE} ] ; then
-    echo "I:checking non-cachable NODATA response handling using dns_client"
+    n=`expr $n + 1`
+    echo "I:checking non-cachable NODATA response handling using dns_client ($n)"
     ret=0
-    ${SAMPLE} -p 5300 -t a 10.53.0.1 nodata.example.net 2> sample.out || ret=1
-    grep "resolution failed: ncache nxrrset" sample.out > /dev/null || ret=1
+    ${SAMPLE} -p 5300 -t a 10.53.0.1 nodata.example.net 2> sample.out.ns1.test${n} || ret=1
+    grep "resolution failed: ncache nxrrset" sample.out.ns1.test${n} > /dev/null || ret=1
     if [ $ret != 0 ]; then echo "I:failed"; fi
     status=`expr $status + $ret`
 fi
 
-echo "I:checking handling of bogus referrals"
+n=`expr $n + 1`
+echo "I:checking handling of bogus referrals ($n)"
 # If the server has the "INSIST(!external)" bug, this query will kill it.
-$DIG +tcp www.example.com. a @10.53.0.1 -p 5300 >/dev/null || status=1
+$DIG +tcp www.example.com. a @10.53.0.1 -p 5300 >/dev/null || { echo I:failed; status=`expr $status + 1`; }
 
 if [ -x ${SAMPLE} ] ; then
-    echo "I:checking handling of bogus referrals using dns_client"
+    n=`expr $n + 1`
+    echo "I:checking handling of bogus referrals using dns_client ($n)"
     ret=0
-    ${SAMPLE} -p 5300 -t a 10.53.0.1 www.example.com 2> sample.out || ret=1
-    grep "resolution failed: failure" sample.out > /dev/null || ret=1
+    ${SAMPLE} -p 5300 -t a 10.53.0.1 www.example.com 2> sample.out.ns1.test${n} || ret=1
+    egrep "resolution failed: (failure|SERVFAIL)" sample.out.ns1.test${n} > /dev/null || ret=1
     if [ $ret != 0 ]; then echo "I:failed"; fi
     status=`expr $status + $ret`
 fi
 
-echo "I:check handling of cname + other data / 1"
-$DIG +tcp cname1.example.com. a @10.53.0.1 -p 5300 >/dev/null || status=1
+n=`expr $n + 1`
+echo "I:check handling of cname + other data / 1 ($n)"
+$DIG +tcp cname1.example.com. a @10.53.0.1 -p 5300 >/dev/null || { echo I:failed; status=`expr $status + 1`; }
 
-echo "I:check handling of cname + other data / 2"
-$DIG +tcp cname2.example.com. a @10.53.0.1 -p 5300 >/dev/null || status=1
+n=`expr $n + 1`
+echo "I:check handling of cname + other data / 2 ($n)"
+$DIG +tcp cname2.example.com. a @10.53.0.1 -p 5300 >/dev/null || { echo I:failed; status=`expr $status + 1`; }
 
-echo "I:check that server is still running"
-$DIG +tcp www.example.com. a @10.53.0.1 -p 5300 >/dev/null || status=1
+n=`expr $n + 1`
+echo "I:check that server is still running ($n)"
+$DIG +tcp www.example.com. a @10.53.0.1 -p 5300 >/dev/null || { echo I:failed; status=`expr $status + 1`; }
 
-echo "I:checking answer IPv4 address filtering (deny)"
+n=`expr $n + 1`
+echo "I:checking answer IPv4 address filtering (deny) ($n)"
 ret=0
-$DIG +tcp www.example.net @10.53.0.1 a -p 5300 > dig.out || ret=1
-grep "status: SERVFAIL" dig.out > /dev/null || ret=1
+$DIG +tcp www.example.net @10.53.0.1 a -p 5300 > dig.out.ns1.test${n} || ret=1
+grep "status: SERVFAIL" dig.out.ns1.test${n} > /dev/null || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
-echo "I:checking answer IPv6 address filtering (deny)"
+n=`expr $n + 1`
+echo "I:checking answer IPv6 address filtering (deny) ($n)"
 ret=0
-$DIG +tcp www.example.net @10.53.0.1 aaaa -p 5300 > dig.out || ret=1
-grep "status: SERVFAIL" dig.out > /dev/null || ret=1
+$DIG +tcp www.example.net @10.53.0.1 aaaa -p 5300 > dig.out.ns1.test${n} || ret=1
+grep "status: SERVFAIL" dig.out.ns1.test${n} > /dev/null || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
-echo "I:checking answer IPv4 address filtering (accept)"
+n=`expr $n + 1`
+echo "I:checking answer IPv4 address filtering (accept) ($n)"
 ret=0
-$DIG +tcp www.example.org @10.53.0.1 a -p 5300 > dig.out || ret=1
-grep "status: NOERROR" dig.out > /dev/null || ret=1
+$DIG +tcp www.example.org @10.53.0.1 a -p 5300 > dig.out.ns1.test${n} || ret=1
+grep "status: NOERROR" dig.out.ns1.test${n} > /dev/null || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
 
 if [ -x ${SAMPLE} ] ; then
-    echo "I:checking answer IPv4 address filtering using dns_client (accept)"
+    n=`expr $n + 1`
+    echo "I:checking answer IPv4 address filtering using dns_client (accept) ($n)"
     ret=0
-    ${SAMPLE} -p 5300 -t a 10.53.0.1 www.example.org > sample.out || ret=1
-    grep "www.example.org..*.192.0.2.1" sample.out > /dev/null || ret=1
+    ${SAMPLE} -p 5300 -t a 10.53.0.1 www.example.org > sample.out.ns1.test${n} || ret=1
+    grep "www.example.org..*.192.0.2.1" sample.out.ns1.test${n} > /dev/null || ret=1
     if [ $ret != 0 ]; then echo "I:failed"; fi
     status=`expr $status + $ret`
 fi
 
-echo "I:checking answer IPv6 address filtering (accept)"
+n=`expr $n + 1`
+echo "I:checking answer IPv6 address filtering (accept) ($n)"
 ret=0
-$DIG +tcp www.example.org @10.53.0.1 aaaa -p 5300 > dig.out || ret=1
-grep "status: NOERROR" dig.out > /dev/null || ret=1
+$DIG +tcp www.example.org @10.53.0.1 aaaa -p 5300 > dig.out.ns1.test${n} || ret=1
+grep "status: NOERROR" dig.out.ns1.test${n} > /dev/null || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
 if [ -x ${SAMPLE} ] ; then
-    echo "I:checking answer IPv6 address filtering using dns_client (accept)"
+    n=`expr $n + 1`
+    echo "I:checking answer IPv6 address filtering using dns_client (accept) ($n)"
     ret=0
-    ${SAMPLE} -p 5300 -t aaaa 10.53.0.1 www.example.org > sample.out || ret=1
-    grep "www.example.org..*.2001:db8:beef::1" sample.out > /dev/null || ret=1
+    ${SAMPLE} -p 5300 -t aaaa 10.53.0.1 www.example.org > sample.out.ns1.test${n} || ret=1
+    grep "www.example.org..*.2001:db8:beef::1" sample.out.ns1.test${n} > /dev/null || ret=1
     if [ $ret != 0 ]; then echo "I:failed"; fi
     status=`expr $status + $ret`
 fi
 
-echo "I:checking CNAME target filtering (deny)"
+n=`expr $n + 1`
+echo "I:checking CNAME target filtering (deny) ($n)"
 ret=0
-$DIG +tcp badcname.example.net @10.53.0.1 a -p 5300 > dig.out || ret=1
-grep "status: SERVFAIL" dig.out > /dev/null || ret=1
+$DIG +tcp badcname.example.net @10.53.0.1 a -p 5300 > dig.out.ns1.test${n} || ret=1
+grep "status: SERVFAIL" dig.out.ns1.test${n} > /dev/null || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
-echo "I:checking CNAME target filtering (accept)"
+n=`expr $n + 1`
+echo "I:checking CNAME target filtering (accept) ($n)"
 ret=0
-$DIG +tcp goodcname.example.net @10.53.0.1 a -p 5300 > dig.out || ret=1
-grep "status: NOERROR" dig.out > /dev/null || ret=1
+$DIG +tcp goodcname.example.net @10.53.0.1 a -p 5300 > dig.out.ns1.test${n} || ret=1
+grep "status: NOERROR" dig.out.ns1.test${n} > /dev/null || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
 if [ -x ${SAMPLE} ] ; then
-    echo "I:checking CNAME target filtering using dns_client (accept)"
+    n=`expr $n + 1`
+    echo "I:checking CNAME target filtering using dns_client (accept) ($n)"
     ret=0
-    ${SAMPLE} -p 5300 -t a 10.53.0.1 goodcname.example.net > sample.out || ret=1
-    grep "goodcname.example.net..*.goodcname.example.org." sample.out > /dev/null || ret=1
-    grep "goodcname.example.org..*.192.0.2.1" sample.out > /dev/null || ret=1
+    ${SAMPLE} -p 5300 -t a 10.53.0.1 goodcname.example.net > sample.out.ns1.test${n} || ret=1
+    grep "goodcname.example.net..*.goodcname.example.org." sample.out.ns1.test${n} > /dev/null || ret=1
+    grep "goodcname.example.org..*.192.0.2.1" sample.out.ns1.test${n} > /dev/null || ret=1
     if [ $ret != 0 ]; then echo "I:failed"; fi
     status=`expr $status + $ret`
 fi
 
-echo "I:checking CNAME target filtering (accept due to subdomain)"
+n=`expr $n + 1`
+echo "I:checking CNAME target filtering (accept due to subdomain) ($n)"
 ret=0
-$DIG +tcp cname.sub.example.org @10.53.0.1 a -p 5300 > dig.out || ret=1
-grep "status: NOERROR" dig.out > /dev/null || ret=1
+$DIG +tcp cname.sub.example.org @10.53.0.1 a -p 5300 > dig.out.ns1.test${n} || ret=1
+grep "status: NOERROR" dig.out.ns1.test${n} > /dev/null || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
 if [ -x ${SAMPLE} ] ; then
-    echo "I:checking CNAME target filtering using dns_client (accept due to subdomain)"
+    n=`expr $n + 1`
+    echo "I:checking CNAME target filtering using dns_client (accept due to subdomain) ($n)"
     ret=0
-    ${SAMPLE} -p 5300 -t a 10.53.0.1 cname.sub.example.org > sample.out || ret=1
-    grep "cname.sub.example.org..*.ok.sub.example.org." sample.out > /dev/null || ret=1
-    grep "ok.sub.example.org..*.192.0.2.1" sample.out > /dev/null || ret=1
+    ${SAMPLE} -p 5300 -t a 10.53.0.1 cname.sub.example.org > sample.out.ns1.test${n} || ret=1
+    grep "cname.sub.example.org..*.ok.sub.example.org." sample.out.ns1.test${n} > /dev/null || ret=1
+    grep "ok.sub.example.org..*.192.0.2.1" sample.out.ns1.test${n} > /dev/null || ret=1
     if [ $ret != 0 ]; then echo "I:failed"; fi
     status=`expr $status + $ret`
 fi
 
-echo "I:checking DNAME target filtering (deny)"
+n=`expr $n + 1`
+echo "I:checking DNAME target filtering (deny) ($n)"
 ret=0
-$DIG +tcp foo.baddname.example.net @10.53.0.1 a -p 5300 > dig.out || ret=1
-grep "status: SERVFAIL" dig.out > /dev/null || ret=1
+$DIG +tcp foo.baddname.example.net @10.53.0.1 a -p 5300 > dig.out.ns1.test${n} || ret=1
+grep "status: SERVFAIL" dig.out.ns1.test${n} > /dev/null || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
-echo "I:checking DNAME target filtering (accept)"
+n=`expr $n + 1`
+echo "I:checking DNAME target filtering (accept) ($n)"
 ret=0
-$DIG +tcp foo.gooddname.example.net @10.53.0.1 a -p 5300 > dig.out || ret=1
-grep "status: NOERROR" dig.out > /dev/null || ret=1
+$DIG +tcp foo.gooddname.example.net @10.53.0.1 a -p 5300 > dig.out.ns1.test${n} || ret=1
+grep "status: NOERROR" dig.out.ns1.test${n} > /dev/null || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
 if [ -x ${SAMPLE} ] ; then
-    echo "I:checking DNAME target filtering using dns_client (accept)"
+    n=`expr $n + 1`
+    echo "I:checking DNAME target filtering using dns_client (accept) ($n)"
     ret=0
-    ${SAMPLE} -p 5300 -t a 10.53.0.1 foo.gooddname.example.net > sample.out || ret=1
-    grep "foo.gooddname.example.net..*.gooddname.example.org" sample.out > /dev/null || ret=1
-    grep "foo.gooddname.example.org..*.192.0.2.1" sample.out > /dev/null || ret=1
+    ${SAMPLE} -p 5300 -t a 10.53.0.1 foo.gooddname.example.net > sample.out.ns1.test${n} || ret=1
+    grep "foo.gooddname.example.net..*.gooddname.example.org" sample.out.ns1.test${n} > /dev/null || ret=1
+    grep "foo.gooddname.example.org..*.192.0.2.1" sample.out.ns1.test${n} > /dev/null || ret=1
     if [ $ret != 0 ]; then echo "I:failed"; fi
     status=`expr $status + $ret`
 fi
 
-echo "I:checking DNAME target filtering (accept due to subdomain)"
+n=`expr $n + 1`
+echo "I:checking DNAME target filtering (accept due to subdomain) ($n)"
 ret=0
-$DIG +tcp www.dname.sub.example.org @10.53.0.1 a -p 5300 > dig.out || ret=1
-grep "status: NOERROR" dig.out > /dev/null || ret=1
+$DIG +tcp www.dname.sub.example.org @10.53.0.1 a -p 5300 > dig.out.ns1.test${n} || ret=1
+grep "status: NOERROR" dig.out.ns1.test${n} > /dev/null || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
 if [ -x ${SAMPLE} ] ; then
-    echo "I:checking DNAME target filtering using dns_client (accept due to subdomain)"
+    n=`expr $n + 1`
+    echo "I:checking DNAME target filtering using dns_client (accept due to subdomain) ($n)"
     ret=0
-    ${SAMPLE} -p 5300 -t a 10.53.0.1 www.dname.sub.example.org > sample.out || ret=1
-    grep "www.dname.sub.example.org..*.ok.sub.example.org." sample.out > /dev/null || ret=1
-    grep "www.ok.sub.example.org..*.192.0.2.1" sample.out > /dev/null || ret=1
+    ${SAMPLE} -p 5300 -t a 10.53.0.1 www.dname.sub.example.org > sample.out.ns1.test${n} || ret=1
+    grep "www.dname.sub.example.org..*.ok.sub.example.org." sample.out.ns1.test${n} > /dev/null || ret=1
+    grep "www.ok.sub.example.org..*.192.0.2.1" sample.out.ns1.test${n} > /dev/null || ret=1
     if [ $ret != 0 ]; then echo "I:failed"; fi
     status=`expr $status + $ret`
 fi
@@ -379,13 +402,177 @@ done
 [ $ret = 0 ] && ret=$foo; 
 if [ $ret != 0 ]; then echo "I:failed"; status=1; fi
 
-echo "I:check for improved error message with SOA mismatch"
+n=`expr $n + 1`
+echo "I:check for improved error message with SOA mismatch ($n)"
 ret=0
-$DIG @10.53.0.1 -p 5300 www.sub.broken aaaa > dig.out.${n} || ret=1
+$DIG @10.53.0.1 -p 5300 www.sub.broken aaaa > dig.out.ns1.test${n} || ret=1
 grep "not subdomain of zone" ns1/named.run > /dev/null || ret=1
 if [ $ret != 0 ]; then echo "I:failed"; fi
 status=`expr $status + $ret`
 
-echo "I:exit status: $status"
+cp ns7/named2.conf ns7/named.conf
+$RNDC -c ../common/rndc.conf -s 10.53.0.7 -p 9953 reconfig 2>&1 | sed 's/^/I:ns7 /'
 
-exit $status
+n=`expr $n + 1`
+echo "I:check resolution on the listening port ($n)"
+ret=0
+$DIG +tcp +tries=2 +time=5 mx example.net @10.53.0.7 -p 5300 > dig.ns7.out.${n} || ret=2
+grep "status: NOERROR" dig.ns7.out.${n} > /dev/null || ret=1
+grep "ANSWER: 1" dig.ns7.out.${n} > /dev/null || ret=1
+if [ $ret != 0 ]; then echo "I:failed"; ret=1; fi
+status=`expr $status + $ret`
+
+n=`expr $n + 1`
+echo "I:check that '-t aaaa' in .digrc does not have unexpected side effects ($n)"
+ret=0
+echo "-t aaaa" > .digrc
+env HOME=`pwd` $DIG @10.53.0.4 -p 5300 . > dig.out.1.${n} || ret=1
+env HOME=`pwd` $DIG @10.53.0.4 -p 5300 . A > dig.out.2.${n} || ret=1
+env HOME=`pwd` $DIG @10.53.0.4 -p 5300 -x 127.0.0.1 > dig.out.3.${n} || ret=1
+grep ';\..*IN.*AAAA$' dig.out.1.${n} > /dev/null || ret=1
+grep ';\..*IN.*A$' dig.out.2.${n} > /dev/null || ret=1
+grep 'extra type option' dig.out.2.${n} > /dev/null && ret=1
+grep ';1\.0\.0\.127\.in-addr\.arpa\..*IN.*PTR$' dig.out.3.${n} > /dev/null || ret=1
+if [ $ret != 0 ]; then echo "I:failed"; fi
+status=`expr $status + $ret`
+
+n=`expr $n + 1`
+echo "I:check that CNAME nameserver is logged correctly (${n})"
+ret=0
+$DIG soa all-cnames @10.53.0.5 -p 5300 > dig.out.ns5.test${n} || ret=1
+grep "status: SERVFAIL" dig.out.ns5.test${n} > /dev/null || ret=1
+grep "skipping nameserver 'cname.tld' because it is a CNAME, while resolving 'all-cnames/SOA'" ns5/named.run > /dev/null || ret=1
+if [ $ret != 0 ]; then echo "I:failed"; fi
+status=`expr $status + $ret`
+
+n=`expr $n + 1`
+echo "I:check that SOA query returns data for delegation-only apex (${n})"
+ret=0
+$DIG soa delegation-only @10.53.0.5 -p 5300 > dig.out.ns5.test${n} || ret=1
+grep "status: NOERROR" dig.out.ns5.test${n} > /dev/null || ret=1
+grep "ANSWER: 1," dig.out.ns5.test${n} > /dev/null || ret=1
+if [ $ret != 0 ]; then echo "I:failed"; fi
+status=`expr $status + $ret`
+n=`expr $n + 1`
+
+n=`expr $n + 1`
+echo "I:check that NS query returns data for delegation-only apex (${n})"
+ret=0
+$DIG ns delegation-only @10.53.0.5 -p 5300 > dig.out.ns5.test${n} || ret=1
+grep "status: NOERROR" dig.out.ns5.test${n} > /dev/null || ret=1
+grep "ANSWER: 1," dig.out.ns5.test${n} > /dev/null || ret=1
+if [ $ret != 0 ]; then echo "I:failed"; fi
+status=`expr $status + $ret`
+
+n=`expr $n + 1`
+echo "I:check that A query returns data for delegation-only A apex (${n})"
+ret=0
+$DIG a delegation-only @10.53.0.5 -p 5300 > dig.out.ns5.test${n} || ret=1
+grep "status: NOERROR" dig.out.ns5.test${n} > /dev/null || ret=1
+grep "ANSWER: 1," dig.out.ns5.test${n} > /dev/null || ret=1
+if [ $ret != 0 ]; then echo "I:failed"; fi
+status=`expr $status + $ret`
+
+n=`expr $n + 1`
+echo "I:check that CDS query returns data for delegation-only apex (${n})"
+ret=0
+$DIG cds delegation-only @10.53.0.5 -p 5300 > dig.out.ns5.test${n} || ret=1
+grep "status: NOERROR" dig.out.ns5.test${n} > /dev/null || ret=1
+grep "ANSWER: 1," dig.out.ns5.test${n} > /dev/null || ret=1
+if [ $ret != 0 ]; then echo "I:failed"; fi
+status=`expr $status + $ret`
+
+n=`expr $n + 1`
+echo "I:check that AAAA query returns data for delegation-only AAAA apex (${n})"
+ret=0
+$DIG a delegation-only @10.53.0.5 -p 5300 > dig.out.ns5.test${n} || ret=1
+grep "status: NOERROR" dig.out.ns5.test${n} > /dev/null || ret=1
+grep "ANSWER: 1," dig.out.ns5.test${n} > /dev/null || ret=1
+if [ $ret != 0 ]; then echo "I:failed"; fi
+status=`expr $status + $ret`
+n=`expr $n + 1`
+
+echo "I:check that DNSKEY query returns data for delegation-only apex (${n})"
+ret=0
+$DIG dnskey delegation-only @10.53.0.5 -p 5300 > dig.out.ns5.test${n} || ret=1
+grep "status: NOERROR" dig.out.ns5.test${n} > /dev/null || ret=1
+grep "ANSWER: 1," dig.out.ns5.test${n} > /dev/null || ret=1
+if [ $ret != 0 ]; then echo "I:failed"; fi
+status=`expr $status + $ret`
+
+n=`expr $n + 1`
+echo "I:check that CDNSKEY query returns data for delegation-only apex (${n})"
+ret=0
+$DIG cdnskey delegation-only @10.53.0.5 -p 5300 > dig.out.ns5.test${n} || ret=1
+grep "status: NOERROR" dig.out.ns5.test${n} > /dev/null || ret=1
+grep "ANSWER: 1," dig.out.ns5.test${n} > /dev/null || ret=1
+if [ $ret != 0 ]; then echo "I:failed"; fi
+status=`expr $status + $ret`
+
+n=`expr $n + 1`
+echo "I:check that NXDOMAIN is returned for delegation-only non-apex A data (${n})"
+ret=0
+$DIG a a.delegation-only @10.53.0.5 -p 5300 > dig.out.ns5.test${n} || ret=1
+grep "status: NXDOMAIN" dig.out.ns5.test${n} > /dev/null || ret=1
+if [ $ret != 0 ]; then echo "I:failed"; fi
+status=`expr $status + $ret`
+
+n=`expr $n + 1`
+echo "I:check that NXDOMAIN is returned for delegation-only non-apex CDS data (${n})"
+ret=0
+$DIG cds cds.delegation-only @10.53.0.5 -p 5300 > dig.out.ns5.test${n} || ret=1
+grep "status: NXDOMAIN" dig.out.ns5.test${n} > /dev/null || ret=1
+if [ $ret != 0 ]; then echo "I:failed"; fi
+status=`expr $status + $ret`
+
+n=`expr $n + 1`
+echo "I:check that NXDOMAIN is returned for delegation-only non-apex AAAA data (${n})"
+ret=0
+$DIG aaaa aaaa.delegation-only @10.53.0.5 -p 5300 > dig.out.ns5.test${n} || ret=1
+grep "status: NXDOMAIN" dig.out.ns5.test${n} > /dev/null || ret=1
+if [ $ret != 0 ]; then echo "I:failed"; fi
+status=`expr $status + $ret`
+n=`expr $n + 1`
+
+echo "I:check that NXDOMAIN is returned for delegation-only non-apex CDNSKEY data (${n})"
+ret=0
+$DIG cdnskey cdnskey.delegation-only @10.53.0.5 -p 5300 > dig.out.ns5.test${n} || ret=1
+grep "status: NXDOMAIN" dig.out.ns5.test${n} > /dev/null || ret=1
+if [ $ret != 0 ]; then echo "I:failed"; fi
+status=`expr $status + $ret`
+
+n=`expr $n + 1`
+echo "I:check zero ttl not returned for learnt non zero ttl records (${n})"
+ret=0
+# use prefetch disabled server
+$DIG @10.53.0.7 -p 5300 non-zero.example.net txt > dig.out.1.${n} || ret=1
+ttl1=`awk '/"A" "short" "ttl"/ { print $2 - 2 }' dig.out.1.${n}`
+# sleep so we are in expire range
+sleep ${ttl1:-0}
+# look for ttl = 1, allow for one miss at getting zero ttl
+zerotonine="0 1 2 3 4 5 6 7 8 9"
+zerotonine="$zerotonine $zerotonine $zerotonine"
+for i in $zerotonine $zerotonine $zerotonine $zerotonine
+do
+	$DIG @10.53.0.7 -p 5300 non-zero.example.net txt > dig.out.2.${n} || ret=1
+	ttl2=`awk '/"A" "short" "ttl"/ { print $2 }' dig.out.2.${n}`
+	test ${ttl2:-1} -eq 0 && break
+	test ${ttl2:-1} -ge ${ttl1:-0} && break
+	$PERL -e 'select(undef, undef, undef, 0.05);'
+done
+test ${ttl2:-1} -eq 0 && ret=1
+test ${ttl2:-1} -ge ${ttl1:-0} || break
+if [ $ret != 0 ]; then echo "I:failed"; fi
+status=`expr $status + $ret`
+
+n=`expr $n + 1`
+echo "I:check zero ttl is returned for learnt zero ttl records (${n})"
+ret=0
+$DIG @10.53.0.7 -p 5300 zero.example.net txt > dig.out.1.${n} || ret=1
+ttl=`awk '/"A" "zero" "ttl"/ { print $2 }' dig.out.1.${n}`
+test ${ttl:-1} -eq 0 || ret=1
+if [ $ret != 0 ]; then echo "I:failed"; fi
+status=`expr $status + $ret`
+
+echo "I:exit status: $status"
+[ $status -eq 0 ] || exit 1
