@@ -1,5 +1,5 @@
 /*
- * Portions Copyright (C) 2004-2007, 2012  Internet Systems Consortium, Inc. ("ISC")
+ * Portions Copyright (C) 2004-2007, 2012, 2013, 2015, 2016  Internet Systems Consortium, Inc. ("ISC")
  * Portions Copyright (C) 2001-2003  Internet Software Consortium.
  *
  * Permission to use, copy, modify, and/or distribute this software for any
@@ -29,8 +29,6 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: cc.c,v 1.18 2007/08/28 07:20:43 tbox Exp $ */
-
 /*! \file */
 
 #include <config.h>
@@ -42,6 +40,7 @@
 #include <isc/assertions.h>
 #include <isc/hmacmd5.h>
 #include <isc/print.h>
+#include <isc/safe.h>
 #include <isc/stdlib.h>
 
 #include <isccc/alist.h>
@@ -84,9 +83,8 @@ static isc_result_t
 list_towire(isccc_sexpr_t *alist, isccc_region_t *target);
 
 static isc_result_t
-value_towire(isccc_sexpr_t *elt, isccc_region_t *target)
-{
-	size_t len;
+value_towire(isccc_sexpr_t *elt, isccc_region_t *target) {
+	unsigned int len;
 	unsigned char *lenp;
 	isccc_region_t *vr;
 	isc_result_t result;
@@ -116,7 +114,7 @@ value_towire(isccc_sexpr_t *elt, isccc_region_t *target)
 		result = table_towire(elt, target);
 		if (result != ISC_R_SUCCESS)
 			return (result);
-		len = (size_t)(target->rstart - lenp);
+		len = (unsigned int)(target->rstart - lenp);
 		/*
 		 * 'len' is 4 bytes too big, since it counts
 		 * the placeholder length too.  Adjust and
@@ -140,7 +138,7 @@ value_towire(isccc_sexpr_t *elt, isccc_region_t *target)
 		result = list_towire(elt, target);
 		if (result != ISC_R_SUCCESS)
 			return (result);
-		len = (size_t)(target->rstart - lenp);
+		len = (unsigned int)(target->rstart - lenp);
 		/*
 		 * 'len' is 4 bytes too big, since it counts
 		 * the placeholder length.  Adjust and emit.
@@ -159,7 +157,7 @@ table_towire(isccc_sexpr_t *alist, isccc_region_t *target)
 	isccc_sexpr_t *kv, *elt, *k, *v;
 	char *ks;
 	isc_result_t result;
-	size_t len;
+	unsigned int len;
 
 	for (elt = isccc_alist_first(alist);
 	     elt != NULL;
@@ -168,7 +166,7 @@ table_towire(isccc_sexpr_t *alist, isccc_region_t *target)
 		k = ISCCC_SEXPR_CAR(kv);
 		ks = isccc_sexpr_tostring(k);
 		v = ISCCC_SEXPR_CDR(kv);
-		len = strlen(ks);
+		len = (unsigned int)strlen(ks);
 		INSIST(len <= 255U);
 		/*
 		 * Emit the key name.
@@ -264,7 +262,8 @@ isccc_cc_towire(isccc_sexpr_t *alist, isccc_region_t *target,
 	if (result != ISC_R_SUCCESS)
 		return (result);
 	if (secret != NULL)
-		return (sign(signed_rstart, (target->rstart - signed_rstart),
+		return (sign(signed_rstart,
+			     (unsigned int)(target->rstart - signed_rstart),
 			     hmd5_rstart, secret));
 	return (ISC_R_SUCCESS);
 }
@@ -280,15 +279,17 @@ verify(isccc_sexpr_t *alist, unsigned char *data, unsigned int length,
 	isccc_sexpr_t *_auth, *hmd5;
 	unsigned char digest[ISC_MD5_DIGESTLENGTH];
 	unsigned char digestb64[ISC_MD5_DIGESTLENGTH * 4];
+	isccc_region_t *region;
+	unsigned char *value;
 
 	/*
 	 * Extract digest.
 	 */
 	_auth = isccc_alist_lookup(alist, "_auth");
-	if (_auth == NULL)
+	if (!isccc_alist_alistp(_auth))
 		return (ISC_R_FAILURE);
 	hmd5 = isccc_alist_lookup(_auth, "hmd5");
-	if (hmd5 == NULL)
+	if (!isccc_sexpr_binaryp(hmd5))
 		return (ISC_R_FAILURE);
 	/*
 	 * Compute digest.
@@ -308,10 +309,15 @@ verify(isccc_sexpr_t *alist, unsigned char *data, unsigned int length,
 	 */
 	target.rstart -= 2;
 	*target.rstart++ = '\0';
+
 	/*
 	 * Verify.
 	 */
-	if (strcmp((char *)digestb64, isccc_sexpr_tostring(hmd5)) != 0)
+	region = isccc_sexpr_tobinary(hmd5);
+	if ((region->rend - region->rstart) != HMD5_LENGTH)
+		return (ISCCC_R_BADAUTH);
+	value = region->rstart;
+	if (!isc_safe_memequal(value, digestb64, HMD5_LENGTH))
 		return (ISCCC_R_BADAUTH);
 
 	return (ISC_R_SUCCESS);
@@ -325,8 +331,7 @@ static isc_result_t
 list_fromwire(isccc_region_t *source, isccc_sexpr_t **listp);
 
 static isc_result_t
-value_fromwire(isccc_region_t *source, isccc_sexpr_t **valuep)
-{
+value_fromwire(isccc_region_t *source, isccc_sexpr_t **valuep) {
 	unsigned int msgtype;
 	isc_uint32_t len;
 	isccc_sexpr_t *value;
@@ -399,11 +404,10 @@ table_fromwire(isccc_region_t *source, isccc_region_t *secret,
 		first_tag = ISC_FALSE;
 	}
 
-	*alistp = alist;
-
 	if (secret != NULL) {
 		if (checksum_rstart != NULL)
 			result = verify(alist, checksum_rstart,
+					(unsigned int)
 					(source->rend - checksum_rstart),
 					secret);
 		else
@@ -412,15 +416,16 @@ table_fromwire(isccc_region_t *source, isccc_region_t *secret,
 		result = ISC_R_SUCCESS;
 
  bad:
-	if (result != ISC_R_SUCCESS)
+	if (result == ISC_R_SUCCESS)
+		*alistp = alist;
+	else
 		isccc_sexpr_free(&alist);
 
 	return (result);
 }
 
 static isc_result_t
-list_fromwire(isccc_region_t *source, isccc_sexpr_t **listp)
-{
+list_fromwire(isccc_region_t *source, isccc_sexpr_t **listp) {
 	isccc_sexpr_t *list, *value;
 	isc_result_t result;
 
@@ -435,7 +440,7 @@ list_fromwire(isccc_region_t *source, isccc_sexpr_t **listp)
 		if (isccc_sexpr_addtolist(&list, value) == NULL) {
 			isccc_sexpr_free(&value);
 			isccc_sexpr_free(&list);
-			return (result);
+			return (ISC_R_NOMEMORY);
 		}
 	}
 
@@ -541,7 +546,7 @@ isccc_cc_createack(isccc_sexpr_t *message, isc_boolean_t ok,
 	REQUIRE(ackp != NULL && *ackp == NULL);
 
 	_ctrl = isccc_alist_lookup(message, "_ctrl");
-	if (_ctrl == NULL ||
+	if (!isccc_alist_alistp(_ctrl) ||
 	    isccc_cc_lookupuint32(_ctrl, "_ser", &serial) != ISC_R_SUCCESS ||
 	    isccc_cc_lookupuint32(_ctrl, "_tim", &t) != ISC_R_SUCCESS)
 		return (ISC_R_FAILURE);
@@ -561,8 +566,10 @@ isccc_cc_createack(isccc_sexpr_t *message, isc_boolean_t ok,
 		return (result);
 
 	_ctrl = isccc_alist_lookup(ack, "_ctrl");
-	if (_ctrl == NULL)
-		return (ISC_R_FAILURE);
+	if (_ctrl == NULL) {
+		result = ISC_R_FAILURE;
+		goto bad;
+	}
 	if (isccc_cc_definestring(ack, "_ack", (ok) ? "1" : "0") == NULL) {
 		result = ISC_R_NOMEMORY;
 		goto bad;
@@ -579,12 +586,11 @@ isccc_cc_createack(isccc_sexpr_t *message, isc_boolean_t ok,
 }
 
 isc_boolean_t
-isccc_cc_isack(isccc_sexpr_t *message)
-{
+isccc_cc_isack(isccc_sexpr_t *message) {
 	isccc_sexpr_t *_ctrl;
 
 	_ctrl = isccc_alist_lookup(message, "_ctrl");
-	if (_ctrl == NULL)
+	if (!isccc_alist_alistp(_ctrl))
 		return (ISC_FALSE);
 	if (isccc_cc_lookupstring(_ctrl, "_ack", NULL) == ISC_R_SUCCESS)
 		return (ISC_TRUE);
@@ -592,12 +598,11 @@ isccc_cc_isack(isccc_sexpr_t *message)
 }
 
 isc_boolean_t
-isccc_cc_isreply(isccc_sexpr_t *message)
-{
+isccc_cc_isreply(isccc_sexpr_t *message) {
 	isccc_sexpr_t *_ctrl;
 
 	_ctrl = isccc_alist_lookup(message, "_ctrl");
-	if (_ctrl == NULL)
+	if (!isccc_alist_alistp(_ctrl))
 		return (ISC_FALSE);
 	if (isccc_cc_lookupstring(_ctrl, "_rpl", NULL) == ISC_R_SUCCESS)
 		return (ISC_TRUE);
@@ -608,7 +613,7 @@ isc_result_t
 isccc_cc_createresponse(isccc_sexpr_t *message, isccc_time_t now,
 		      isccc_time_t expires, isccc_sexpr_t **alistp)
 {
-	char *_frm, *_to, *type;
+	char *_frm, *_to, *type = NULL;
 	isc_uint32_t serial;
 	isccc_sexpr_t *alist, *_ctrl, *_data;
 	isc_result_t result;
@@ -617,8 +622,7 @@ isccc_cc_createresponse(isccc_sexpr_t *message, isccc_time_t now,
 
 	_ctrl = isccc_alist_lookup(message, "_ctrl");
 	_data = isccc_alist_lookup(message, "_data");
-	if (_ctrl == NULL ||
-	    _data == NULL ||
+	if (!isccc_alist_alistp(_ctrl) || !isccc_alist_alistp(_data) ||
 	    isccc_cc_lookupuint32(_ctrl, "_ser", &serial) != ISC_R_SUCCESS ||
 	    isccc_cc_lookupstring(_data, "type", &type) != ISC_R_SUCCESS)
 		return (ISC_R_FAILURE);
@@ -637,26 +641,37 @@ isccc_cc_createresponse(isccc_sexpr_t *message, isccc_time_t now,
 					 &alist);
 	if (result != ISC_R_SUCCESS)
 		return (result);
+
 	_ctrl = isccc_alist_lookup(alist, "_ctrl");
-	if (_ctrl == NULL)
-		return (ISC_R_FAILURE);
+	if (_ctrl == NULL) {
+		result = ISC_R_FAILURE;
+		goto bad;
+	}
+
 	_data = isccc_alist_lookup(alist, "_data");
-	if (_data == NULL)
-		return (ISC_R_FAILURE);
+	if (_data == NULL) {
+		result = ISC_R_FAILURE;
+		goto bad;
+	}
+
 	if (isccc_cc_definestring(_ctrl, "_rpl", "1") == NULL ||
-	    isccc_cc_definestring(_data, "type", type) == NULL) {
-		isccc_sexpr_free(&alist);
-		return (ISC_R_NOMEMORY);
+	    isccc_cc_definestring(_data, "type", type) == NULL)
+	{
+		result = ISC_R_NOMEMORY;
+		goto bad;
 	}
 
 	*alistp = alist;
 
 	return (ISC_R_SUCCESS);
+
+ bad:
+	isccc_sexpr_free(&alist);
+	return (result);
 }
 
 isccc_sexpr_t *
-isccc_cc_definestring(isccc_sexpr_t *alist, const char *key, const char *str)
-{
+isccc_cc_definestring(isccc_sexpr_t *alist, const char *key, const char *str) {
 	size_t len;
 	isccc_region_t r;
 
@@ -668,8 +683,7 @@ isccc_cc_definestring(isccc_sexpr_t *alist, const char *key, const char *str)
 }
 
 isccc_sexpr_t *
-isccc_cc_defineuint32(isccc_sexpr_t *alist, const char *key, isc_uint32_t i)
-{
+isccc_cc_defineuint32(isccc_sexpr_t *alist, const char *key, isc_uint32_t i) {
 	char b[100];
 	size_t len;
 	isccc_region_t r;
@@ -683,10 +697,11 @@ isccc_cc_defineuint32(isccc_sexpr_t *alist, const char *key, isc_uint32_t i)
 }
 
 isc_result_t
-isccc_cc_lookupstring(isccc_sexpr_t *alist, const char *key, char **strp)
-{
+isccc_cc_lookupstring(isccc_sexpr_t *alist, const char *key, char **strp) {
 	isccc_sexpr_t *kv, *v;
 
+	REQUIRE(strp == NULL || *strp == NULL);
+
 	kv = isccc_alist_assq(alist, key);
 	if (kv != NULL) {
 		v = ISCCC_SEXPR_CDR(kv);
@@ -735,9 +750,7 @@ symtab_undefine(char *key, unsigned int type, isccc_symvalue_t value,
 }
 
 static isc_boolean_t
-symtab_clean(char *key, unsigned int type, isccc_symvalue_t value,
-	     void *arg)
-{
+symtab_clean(char *key, unsigned int type, isccc_symvalue_t value, void *arg) {
 	isccc_time_t *now;
 
 	UNUSED(key);
@@ -753,21 +766,18 @@ symtab_clean(char *key, unsigned int type, isccc_symvalue_t value,
 }
 
 isc_result_t
-isccc_cc_createsymtab(isccc_symtab_t **symtabp)
-{
+isccc_cc_createsymtab(isccc_symtab_t **symtabp) {
 	return (isccc_symtab_create(11897, symtab_undefine, NULL, ISC_FALSE,
 				  symtabp));
 }
 
 void
-isccc_cc_cleansymtab(isccc_symtab_t *symtab, isccc_time_t now)
-{
+isccc_cc_cleansymtab(isccc_symtab_t *symtab, isccc_time_t now) {
 	isccc_symtab_foreach(symtab, symtab_clean, &now);
 }
 
 static isc_boolean_t
-has_whitespace(const char *str)
-{
+has_whitespace(const char *str) {
 	char c;
 
 	if (str == NULL)
@@ -785,7 +795,7 @@ isccc_cc_checkdup(isccc_symtab_t *symtab, isccc_sexpr_t *message,
 {
 	const char *_frm;
 	const char *_to;
-	char *_ser, *_tim, *tmp;
+	char *_ser = NULL, *_tim = NULL, *tmp;
 	isc_result_t result;
 	char *key;
 	size_t len;
@@ -793,17 +803,23 @@ isccc_cc_checkdup(isccc_symtab_t *symtab, isccc_sexpr_t *message,
 	isccc_sexpr_t *_ctrl;
 
 	_ctrl = isccc_alist_lookup(message, "_ctrl");
-	if (_ctrl == NULL ||
+	if (!isccc_alist_alistp(_ctrl) ||
 	    isccc_cc_lookupstring(_ctrl, "_ser", &_ser) != ISC_R_SUCCESS ||
 	    isccc_cc_lookupstring(_ctrl, "_tim", &_tim) != ISC_R_SUCCESS)
 		return (ISC_R_FAILURE);
+
+	INSIST(_ser != NULL);
+	INSIST(_tim != NULL);
+
 	/*
 	 * _frm and _to are optional.
 	 */
+	tmp = NULL;
 	if (isccc_cc_lookupstring(_ctrl, "_frm", &tmp) != ISC_R_SUCCESS)
 		_frm = "";
 	else
 		_frm = tmp;
+	tmp = NULL;
 	if (isccc_cc_lookupstring(_ctrl, "_to", &tmp) != ISC_R_SUCCESS)
 		_to = "";
 	else
